/*
 * Callback on funktio joka on asynkronisen funktion parametrina ja joka
 * suoritetaan kun asynkroninen funktio on suoritettu. Callbackit soveltuvat
 * käsittelemään one-off eventtejä eli yhden kerran tapahtuvia tapahtumia kuten
 * esim. datan haku tiedostosta tai kannasta.
 *
 * On hyvä välttää useita sisäkkäisiä callback -funktioita,
 * jotta koodin uudelleenkäyttö ja testaus olisi helpompaa.
 *
 * Seuraava esimerkki luo serverin, hakee JSON-tiedoston ja
 * laittaa sen sisältämän datan templaattiin joka esitetään html-sivulla.
 * Tässä on vältetty sisäkkäsiä calbackkejä ja return-lauseet
 * lopettavat koodin suorituksen jos tulee virhe.
 *
 * aja komennolla: node renderpage.js
 *
 */
//Käytetään Noden core-kirjastoja http ja fs.
const http = require('http');
const fs = require('fs');

//serverin luontiin käytetään callbackkiä
// eslint-disable-next-line no-unused-vars
const server = http
  .createServer(function (req, res) {
    //callback1
    getTitles(res);
  })
  .listen(8003, '127.0.0.1');
console.log('Server Started at port 8003');

//Tiedostojen polut.
const titlesFile = __dirname + '/titles.json';
const templateFile = __dirname + '/template.html';
//const titlesFile = __dirname + '\\titles.json'; //Windowsissa polussa kenoviiva
//const templateFile = __dirname + '\\template.html'; //Windowsissa polussa kenoviiva

function getTitles(res) {
  /* Tapahtuma on titles.json -tiedoston luku muistiin. Callback käsittelee
   * tapahtuman. Jos tapahtuma epäonnistuu, palautetaan handleError -metodi, jos
   * tapahtuma onnistuu, otetaan data tiedostosta ja mennään getTemplate
   * -metodiin
   */
  fs.readFile(titlesFile, function (err, data) {
    //callback2
    if (err) {
      return handleError(err, res); //jos tulee virhe, poistutaan returnilla handleErroriin
    }
    getTemplate(JSON.parse(data.toString()), res);
  });
}
function getTemplate(titles, res) {
  //templaatin luku muistiin on toinen tapahtuma jonka callback käsittelee
  fs.readFile(templateFile, function (err, data) {
    //callback3
    if (err) {
      return handleError(err, res);
    }
    formatHtml(titles, data.toString(), res);
  });
}
//Muodostetaan HTML-sivu datasta ja templaatista.
function formatHtml(titles, tmpl, res) {
  const html = tmpl.replace('%', titles.join('</li><li>'));
  res.writeHead(200, { 'Content-Type': 'text/html' });
  res.end(html);
}
function handleError(err, res) {
  console.error(err);
  res.end('Server Error');
}

//Tämäkin toimii, mutta sisäkkäiset callbackit eivät ole hyvä juttu
/*
var http = require('http');
var fs = require('fs');
http.createServer(function (req, res) {
    if (req.url === '/') {
        fs.readFile(titlesFile, function (err, data) {
            if (err) {
                console.error(err);
                res.end('Server Error');
            }
            else {
                var titles = JSON.parse(data.toString());
                fs.readFile(templateFile, function (err, data) {
                    if (err) {
                        console.error(err);
                        res.end('Server Error');
                    }
                    else {
                        var tmpl = data.toString();
                        var html = tmpl.replace('%', titles.join('</li><li>'));
                        res.writeHead(200, {'Content-Type': 'text/html'});
                        res.end(html);
                    }
                });
            }
        });
    }
}).listen(8000, "127.0.0.1");
*/
